package xyz.lychee.lagfixer.libs.adventure.title;

import java.time.Duration;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.util.Ticks;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.UnknownNullability;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.NonExtendable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.ScheduledForRemoval;
import xyz.lychee.lagfixer.libs.examination.Examinable;

@NonExtendable
public interface Title extends Examinable {
   Times DEFAULT_TIMES = Title.Times.times(Ticks.duration(10L), Ticks.duration(70L), Ticks.duration(20L));

   static @NotNull Title title(final @NotNull Component title, final @NotNull Component subtitle) {
      return title(title, subtitle, DEFAULT_TIMES);
   }

   static @NotNull Title title(final @NotNull Component title, final @NotNull Component subtitle, @Nullable final @Nullable Times times) {
      return new TitleImpl(title, subtitle, times);
   }

   @NotNull Component title();

   @NotNull Component subtitle();

   @Nullable
   @Nullable Times times();

   <T> @UnknownNullability T part(final @NotNull TitlePart<T> part);

   public interface Times extends Examinable {
      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      @NotNull
      static @NotNull Times of(final @NotNull Duration fadeIn, final @NotNull Duration stay, final @NotNull Duration fadeOut) {
         return times(fadeIn, stay, fadeOut);
      }

      @NotNull
      static @NotNull Times times(final @NotNull Duration fadeIn, final @NotNull Duration stay, final @NotNull Duration fadeOut) {
         return new TitleImpl.TimesImpl(fadeIn, stay, fadeOut);
      }

      @NotNull Duration fadeIn();

      @NotNull Duration stay();

      @NotNull Duration fadeOut();
   }
}
